@@ -2,8 +2,9 @@ import re
 import tempfile
 from datetime import timedelta
 from decimal import Decimal
-
+import subprocess
 from django.conf import settings
+import xml.dom.minidom
 
 
 def format_full_name(first_name: str, last_name: str, max_length: int = 20):
@@ -50,10 +51,11 @@ def format_full_name(first_name: str, last_name: str, max_length: int = 20):
     return full_name
 
 
-def format_timedelta(dt: timedelta) -> str:
+def format_timedelta(dt: timedelta, include_seconds: bool = True) -> str:
     """
-    Formats timedelta to readable format, e.g. 1h30min.
+    Formats timedelta to readable format, e.g. 1h30min15s.
     :param dt: timedelta
+    :param include_seconds: If seconds should be included in formatted string. Default is True.
     :return: str
     """
     seconds = int(dt.total_seconds())
@@ -67,6 +69,8 @@ def format_timedelta(dt: timedelta) -> str:
         s += str(hours) + "h"
     if minutes > 0:
         s += str(minutes) + "min"
+    if seconds > 0 and include_seconds:
+        s += str(seconds) + "s"
     if s == "":
         s = "0min"
     return s
@@ -81,8 +85,6 @@ def format_xml(content: str or bytes, encoding: str = 'UTF-8', exceptions: bool
     :param exceptions: Raise exceptions on error
     :return: str (Formatted XML str)
     """
-    import subprocess
-    import xml.dom.minidom
     try:
         if hasattr(settings, 'XMLLINT_PATH') and settings.XMLLINT_PATH:
             if isinstance(content, str):
@@ -110,8 +112,6 @@ def format_xml_bytes(content: str or bytes, encoding: str = 'UTF-8', exceptions:
     :param exceptions: Raise exceptions on error
     :return: bytes (Formatted XML as bytes)
     """
-    import subprocess
-    import xml.dom.minidom
     try:
         if hasattr(settings, 'XMLLINT_PATH') and settings.XMLLINT_PATH:
             if isinstance(content, str):
@@ -141,20 +141,18 @@ def format_xml_file(full_path: str, encoding: str = 'UTF-8', exceptions: bool =
     :param exceptions: Raise exceptions on error
     :return: bytes
     """
-    import subprocess
-    import xml.dom.minidom
     try:
         if hasattr(settings, 'XMLLINT_PATH') and settings.XMLLINT_PATH:
             return subprocess.check_output([settings.XMLLINT_PATH, '--format', full_path])
         with open(full_path, 'rb') as fp:
             return xml.dom.minidom.parse(fp).toprettyxml(encoding=encoding)
-    except Exception as e:
+    except Exception:
         if exceptions:
             raise
     try:
         with open(full_path, 'rb') as fp:
             return fp.read()
-    except Exception as e:
+    except Exception:
         pass
     return b''
 
