@@ -1,35 +1,42 @@
-import json
-import os
 from datetime import datetime, timedelta, date
+from decimal import Decimal
 from os.path import join
-from pprint import pprint
 import pytz
 from django.conf import settings
+from django.contrib.admin.models import LogEntry
+from django.contrib import admin
+from django.contrib.auth.admin import UserAdmin
 from django.contrib.auth.models import User
 from django.core.exceptions import ValidationError
+from django.core.management import CommandParser
 from django.test import TestCase
-from django.utils.timezone import now
-from django.utils.translation import override, ugettext as _
-from jutil.command import get_date_range_by_name
+from django.utils.translation import override, gettext as _
+from jutil.admin import admin_log, admin_obj_url, admin_obj_link
+from jutil.command import get_date_range_by_name, add_date_range_arguments, parse_date_range_arguments
 from jutil.dict import dict_to_html
+from jutil.model import is_model_field_changed, clone_model, get_model_field_label_and_value, get_object_or_none
 from jutil.request import get_ip_info
+from jutil.sftp import parse_sftp_connection
+from jutil.testing import DefaultTestSetupMixin
 from jutil.urls import url_equals, url_mod, url_host
 from jutil.xml import xml_to_dict, dict_to_element
 from rest_framework.test import APIClient
 from jutil.dates import add_month, per_delta, per_month, this_week, next_month, next_week, this_month, last_month, \
     last_year, last_week, yesterday
-from jutil.format import format_full_name, format_xml, format_xml_bytes
+from jutil.format import format_full_name, format_xml, format_xml_bytes, format_timedelta, dec1, dec2, dec3, dec4, dec5, \
+    dec6
 from jutil.parse import parse_datetime, parse_bool
 from jutil.validators import fi_payment_reference_number, se_ssn_validator, se_ssn_filter, fi_iban_validator, \
-    se_iban_validator, iban_filter_readable, email_filter, iban_validator, iban_bank_info, fi_company_reg_id_validator, \
+    se_iban_validator, iban_filter_readable, email_filter, iban_validator, iban_bank_info, fi_company_org_id_validator, \
     email_validator, fi_payment_reference_validator, iso_payment_reference_validator, fi_ssn_age, \
     se_clearing_code_bank_info, ascii_filter, ee_iban_validator, be_iban_validator, dk_iban_validator, \
-    dk_iban_bank_info, dk_clearing_code_bank_name, country_code_sanitizer, phone_sanitizer, email_sanitizer
+    dk_iban_bank_info, dk_clearing_code_bank_name, country_code_sanitizer, phone_sanitizer, email_sanitizer, \
+    fi_company_org_id_generator
 
 
-class Tests(TestCase):
+class Tests(TestCase, DefaultTestSetupMixin):
     def setUp(self):
-        pass
+        self.add_test_user()
 
     def tearDown(self):
         pass
@@ -276,7 +283,7 @@ class Tests(TestCase):
         self.assertEqual(inf[0], 'BPOTBEB1')
         self.assertEqual(inf[1], 'bpost bank')
 
-    def test_reg_id_fi(self):
+    def test_org_id_fi(self):
         valids = [
             'FI01098230',
             'FI-01098230',
@@ -287,17 +294,21 @@ class Tests(TestCase):
             '2084069-1',
         ]
         for valid in valids:
-            # print('test_reg_id_fi:', valid, 'should be valid...', end=' ')
-            fi_company_reg_id_validator(valid)
+            # print('test_org_id_fi:', valid, 'should be valid...', end=' ')
+            fi_company_org_id_validator(valid)
             # print('ok')
         for invalid in invalids:
             try:
-                # print('test_reg_id_fi:', invalid, 'should be invalid', end=' ')
-                fi_company_reg_id_validator(invalid)
+                # print('test_org_id_fi:', invalid, 'should be invalid', end=' ')
+                fi_company_org_id_validator(invalid)
                 self.assertTrue(False)
             except ValidationError:
                 # print('ok')
                 pass
+        for n in range(10):
+            v0 = fi_company_org_id_generator()
+            # print(v0)
+            fi_company_org_id_validator(v0)
 
     def test_reference_number_validators(self):
         valid_fi_refs = [
@@ -372,7 +383,9 @@ class Tests(TestCase):
         from rest_framework.exceptions import ValidationError
 
         with override('fi'):
-            msg = _("'%(value)s' value has an invalid format. It must be in YYYY-MM-DD HH:MM[:ss[.uuuuuu]][TZ] format.")
+            msg = _("“%(value)s” value has an invalid format. It must be in YYYY-MM-DD HH:MM[:ss[.uuuuuu]][TZ] format.")
+            if 'muoto ei kelpaa' not in msg:
+                print(msg)
             self.assertTrue('muoto ei kelpaa' in msg)
 
             try:
@@ -395,10 +408,78 @@ class Tests(TestCase):
         self.assertEqual(dict_to_html(a), res)
 
     def test_format_xml(self):
-        assert settings.XMLLINT_PATH
+        assert settings.XMLLINT_PATH, 'add e.g. XMLLINT_PATH = "/usr/bin/xmllint" to settings.py'
         src = '<ApplicationRequest> <CustomerId>1</CustomerId>  <Command>DownloadFileList</Command><Timestamp>2019-11-27T04:32:18.613452+02:00</Timestamp><Environment>PRODUCTION</Environment></ApplicationRequest>'
         dst_ref = '<?xml version="1.0"?>\n<ApplicationRequest>\n  <CustomerId>1</CustomerId>\n  <Command>DownloadFileList</Command>\n  <Timestamp>2019-11-27T04:32:18.613452+02:00</Timestamp>\n  <Environment>PRODUCTION</Environment>\n</ApplicationRequest>\n'
         dst = format_xml(src)
         self.assertEqual(dst, dst_ref)
         dst = format_xml_bytes(src)
         self.assertEqual(dst, dst_ref.encode())
+
+    def test_parse_sftp(self):
+        test_cases = [
+            ('jani@kajala.com', ['jani', '', 'kajala.com', '']),
+            ('jani.kajala:1231!@kajala.com', ['jani.kajala', '1231!', 'kajala.com', '']),
+            ('jani:1231!@kajala.com:/my/dir', ['jani', '1231!', 'kajala.com', '/my/dir']),
+            ('jani.kajala:1231!@kajala.com:my/dir', ['jani.kajala', '1231!', 'kajala.com', 'my/dir']),
+            ('user=jani;host=kajala.com', ['jani', '', 'kajala.com', '']),
+            ('user=jani.kajala;pass=1231!;host=kajala.com', ['jani.kajala', '1231!', 'kajala.com', '']),
+            ('user=jani;pass=1231!;host=kajala.com;path=/my/dir', ['jani', '1231!', 'kajala.com', '/my/dir']),
+            ('user=jani.kajala;pass=1231!;host=kajala.com;path=my/dir', ['jani.kajala', '1231!', 'kajala.com', 'my/dir']),
+        ]
+        for connection, ref_res in test_cases:
+            res = parse_sftp_connection(connection)
+            self.assertListEqual(list(res), ref_res, 'SFTP connection string "{}" parsed incorrectly'.format(connection))
+
+    def test_admin(self):
+        obj = self.user
+        admin_log([obj], 'Hello, world')
+        e = LogEntry.objects.all().filter(object_id=obj.id).last()
+        self.assertIsNotNone(e)
+        assert isinstance(e, LogEntry)
+        self.assertEqual(e.change_message, 'Hello, world')
+        self.assertEqual(admin_obj_url(obj, 'admin:auth_user_change'), '/admin/auth/user/{}/change/'.format(obj.id))
+        link = admin_obj_link(obj, 'User', 'admin:auth_user_change')
+        self.assertEqual(link, "<a href='/admin/auth/user/{}/change/'>User</a>".format(obj.id))
+
+    def test_cmd_parser(self):
+        parser = CommandParser()
+        add_date_range_arguments(parser)
+        argv = parser.parse_args(['--begin', '2019-06-25', '--end', '2020-02-01'])
+        options = argv.__dict__
+        begin, end, steps = parse_date_range_arguments(options)
+        self.assertEqual(begin, pytz.utc.localize(datetime(2019, 6, 25)))
+        self.assertEqual(end, pytz.utc.localize(datetime(2020, 2, 1)))
+
+    def test_format_timedelta(self):
+        self.assertEqual(format_timedelta(timedelta(seconds=90), include_seconds=True), '1min30s')
+        self.assertEqual(format_timedelta(timedelta(seconds=90), include_seconds=False), '1min')
+
+    def test_dec123456(self):
+        self.assertEqual(dec1(Decimal('1.2345678')), Decimal('1.2'))
+        self.assertEqual(dec2(Decimal('1.2345678')), Decimal('1.23'))
+        self.assertEqual(dec3(Decimal('1.2345678')), Decimal('1.235'))
+        self.assertEqual(dec4(Decimal('1.2345678')), Decimal('1.2346'))
+        self.assertEqual(dec5(Decimal('1.2345678')), Decimal('1.23457'))
+        self.assertEqual(dec6(Decimal('1.2345678')), Decimal('1.234568'))
+
+    def test_model_funcs(self):
+        admin_log([self.user], 'test msg 1')
+        obj = LogEntry.objects.all().order_by('-pk').last()
+        assert isinstance(obj, LogEntry)
+        self.assertFalse(is_model_field_changed(obj, 'change_message'))
+        obj.change_message = 'hello world'
+        self.assertTrue(is_model_field_changed(obj, 'change_message'))
+        obj.save()
+        self.assertFalse(is_model_field_changed(obj, 'change_message'))
+        obj2 = clone_model(obj)
+        assert isinstance(obj2, LogEntry)
+        self.assertEqual(obj.change_message, obj2.change_message)
+        self.assertGreater(obj2.pk, obj.pk)
+        label, val = get_model_field_label_and_value(obj, 'action_time')
+        self.assertEqual(label, _('action time'))
+        self.assertEqual(str(obj.action_time), val)
+        obj_b = get_object_or_none(obj.__class__, id=obj.id)
+        self.assertEqual(obj_b.id, obj.id)
+        obj_b = get_object_or_none(obj.__class__, id=-1)
+        self.assertIsNone(obj_b)
